/**
* LogBox appender providing asynchronous HTTP
*/
component extends="coldbox.system.logging.AbstractAppender" {
	
	property name = "uuid";
	
	/**
	* Constructor
	* @param	name				The unique name for this appender.
	* @param	properties			A map of configuration properties for the appender
	* @param	layout				The layout class to use in this appender for custom message rendering.
	* @param	levelMin			The default log level for this appender, by default it is 0. Optional. ex: LogBox.logLevels.WARN
	* @param	levelMax			The default log level for this appender, by default it is 4. Optional. ex: LogBox.logLevels.WARN
	*/
	HTTPAppender function init(
		required string name,
		struct properties = {},
		string layout = "",
		numeric levelMin = 0,
		numeric levelMax = 4
	){
		// Init supertype
		super.init( argumentCollection = arguments );
		
		// UUID generator
		uuid = createobject( "java", "java.util.UUID" );
		
		// Verify properties
		if( NOT propertyExists( "url" ) ){ 
			throw( message = "No url property defined", type = "HTTPAppender.InvalidProperty" ); 
		}
		
		if( NOT propertyExists( "verb" ) ){ 
			setProperty( "verb", "get" );
		}
		
		if( NOT arrayFindNoCase( ["get", "post", "put", "update", "delete"], getProperty( "verb" ) ) ){
			throw( message = "Property verb incorrect", type = "HTTPAppender.InvalidProperty" );
		}
		
		return this;
	}
	
	/**
	* Runs on registration
	*/
	public void function onRegistration(){
		
	}
	
	/**
	* Pass an entry to HTTP request
	*/
	public void function logMessage( required any logEvent ){
		var verb = lCase( getProperty( "verb" ) );
		var url = getProperty( "url" );
		var body = arguments.logEvent.getMessage();
		var extraInfo = arguments.logEvent.getExtraInfo();
		var httpParams = [];
		if ( structKeyExists( extraInfo, "httpParams" ) ){
			httpParams = extraInfo.httpParams;
			if ( !isArray(httpParams) ){
				throw( message = "extraInfo.httpParams not an array.", type = "HTTPAppender.InvalidType" );
			}
		}
		
		
		// httpService
		var httpService = new http();
		httpService.setMethod( verb );
		httpService.setUrl( url );

		httpService.addParam(
			type = "body",
			value = body
		);
		
		for( var httpParam in httpParams ){
			httpService.addParam( argumentCollection = httpParam );
		}
		httpService.send();
	}
	
}